package net.locusworks.discord.eighttrack.audio;

import java.lang.reflect.Field;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.function.Consumer;
import com.sedmelluq.discord.lavaplayer.player.AudioPlayerManager;
import com.sedmelluq.discord.lavaplayer.player.DefaultAudioPlayer;
import com.sedmelluq.discord.lavaplayer.player.DefaultAudioPlayerManager;
import com.sedmelluq.discord.lavaplayer.source.AudioSourceManagers;
import com.sedmelluq.discord.lavaplayer.track.AudioTrack;
import com.sedmelluq.discord.lavaplayer.track.AudioTrackEndReason;

public class TrackManager extends DefaultAudioPlayer  {
  
  private LinkedBlockingDeque<String> queue;
  private AudioPlayerManager playerManager;
  private TrackScheduler trackScheduler;
  private String currentItem;
  
  public TrackManager() {
    super(new DefaultAudioPlayerManager());
    this.queue = new LinkedBlockingDeque<String>();
    this.trackScheduler = new TrackScheduler();
    
    addListener(trackScheduler);
    
    try {
        Field f = this.getClass().getSuperclass().getDeclaredField("manager");
        f.setAccessible(true);
        this.playerManager = (AudioPlayerManager) f.get(this);
    } catch (SecurityException | NoSuchFieldException | IllegalArgumentException | IllegalAccessException e) {
      throw new IllegalArgumentException(e);
    }
    
    AudioSourceManagers.registerLocalSource(this.playerManager);
  }
  
  public void queueLast(String track) {
    queue.addLast(track);
  }
  
  public void queueNext(String track) {
    queue.addFirst(track);
  }
  
  public boolean hasTracks() {
    return queue.peek() != null && !queue.isEmpty();
  }
  
  public String getCurrentItem() {
    return currentItem;
  }
  
  public AudioTrack play() {
    if (!queue.isEmpty()) {
      try {
        currentItem = queue.poll();
        playerManager.loadItem(currentItem, trackScheduler).get();
        return trackScheduler.getNextTrack();
      } catch (InterruptedException | ExecutionException e) {}
    }
    return null;
  }
  
  public void setFinishedCallback(Consumer<AudioTrackEndReason> callback) {
    trackScheduler.setFinishedCallback(callback);
  }
  
  public void clearTracks() {
    queue.clear();
    trackScheduler.clearTracks();
  }
}
